const LANGSPEC = r'''
{
  "EvalMaxVersion": 1,
  "LogicSigVersion": 1,
  "Ops": [
    {
      "Opcode": 0,
      "Name": "err",
      "Cost": 1,
      "Size": 1,
      "Doc": "Error. Panic immediately. This is primarily a fencepost against accidental zero bytes getting compiled into programs.",
      "Groups": [
        "Flow Control"
      ]
    },
    {
      "Opcode": 1,
      "Name": "sha256",
      "Args": "B",
      "Returns": "B",
      "Cost": 7,
      "Size": 1,
      "Doc": "SHA256 hash of value X, yields [32]byte",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 2,
      "Name": "keccak256",
      "Args": "B",
      "Returns": "B",
      "Cost": 26,
      "Size": 1,
      "Doc": "Keccak256 hash of value X, yields [32]byte",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 3,
      "Name": "sha512_256",
      "Args": "B",
      "Returns": "B",
      "Cost": 9,
      "Size": 1,
      "Doc": "SHA512_256 hash of value X, yields [32]byte",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 4,
      "Name": "ed25519verify",
      "Args": "BBB",
      "Returns": "U",
      "Cost": 1900,
      "Size": 1,
      "Doc": "for (data A, signature B, pubkey C) verify the signature of (\"ProgData\" || program_hash || data) against the pubkey =\u003e {0 or 1}",
      "DocExtra": "The 32 byte public key is the last element on the stack, preceeded by the 64 byte signature at the second-to-last element on the stack, preceeded by the data which was signed at the third-to-last element on the stack.",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 8,
      "Name": "+",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A plus B. Panic on overflow.",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 9,
      "Name": "-",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A minus B. Panic if B \u003e A.",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 10,
      "Name": "/",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A divided by B. Panic if B == 0.",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 11,
      "Name": "*",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A times B. Panic on overflow.",
      "DocExtra": "Overflow is an error condition which halts execution and fails the transaction. Full precision is available from `mulw`.",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 12,
      "Name": "\u003c",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A less than B =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 13,
      "Name": "\u003e",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A greater than B =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 14,
      "Name": "\u003c=",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A less than or equal to B =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 15,
      "Name": "\u003e=",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A greater than or equal to B =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 16,
      "Name": "\u0026\u0026",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A is not zero and B is not zero =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 17,
      "Name": "||",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A is not zero or B is not zero =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 18,
      "Name": "==",
      "Args": "..",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A is equal to B =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 19,
      "Name": "!=",
      "Args": "..",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A is not equal to B =\u003e {0 or 1}",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 20,
      "Name": "!",
      "Args": "U",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "X == 0 yields 1; else 0",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 21,
      "Name": "len",
      "Args": "B",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "yields length of byte value X",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 22,
      "Name": "itob",
      "Args": "U",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "converts uint64 X to big endian bytes",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 23,
      "Name": "btoi",
      "Args": "B",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "converts bytes X as big endian to uint64",
      "DocExtra": "`btoi` panics if the input is longer than 8 bytes",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 24,
      "Name": "%",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A modulo B. Panic if B == 0.",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 25,
      "Name": "|",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A bitwise-or B",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 26,
      "Name": "\u0026",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A bitwise-and B",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 27,
      "Name": "^",
      "Args": "UU",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "A bitwise-xor B",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 28,
      "Name": "~",
      "Args": "U",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "bitwise invert value X",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 29,
      "Name": "mulw",
      "Args": "UU",
      "Returns": "UU",
      "Cost": 1,
      "Size": 1,
      "Doc": "A times B out to 128-bit long result as low (top) and high uint64 values on the stack",
      "Groups": [
        "Arithmetic"
      ]
    },
    {
      "Opcode": 32,
      "Name": "intcblock",
      "Cost": 1,
      "Size": 0,
      "Doc": "load block of uint64 constants",
      "DocExtra": "`intcblock` loads following program bytes into an array of integer constants in the evaluator. These integer constants can be referred to by `intc` and `intc_*` which will push the value onto the stack. Subsequent calls to `intcblock` reset and replace the integer constants available to the script.",
      "ImmediateNote": "{varuint length} [{varuint value}, ...]",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 33,
      "Name": "intc",
      "Returns": "U",
      "Cost": 1,
      "Size": 2,
      "Doc": "push value from uint64 constants to stack by index into constants",
      "ImmediateNote": "{uint8 int constant index}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 34,
      "Name": "intc_0",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 0 from intcblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 35,
      "Name": "intc_1",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 1 from intcblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 36,
      "Name": "intc_2",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 2 from intcblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 37,
      "Name": "intc_3",
      "Returns": "U",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 3 from intcblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 38,
      "Name": "bytecblock",
      "Cost": 1,
      "Size": 0,
      "Doc": "load block of byte-array constants",
      "DocExtra": "`bytecblock` loads the following program bytes into an array of byte string constants in the evaluator. These constants can be referred to by `bytec` and `bytec_*` which will push the value onto the stack. Subsequent calls to `bytecblock` reset and replace the bytes constants available to the script.",
      "ImmediateNote": "{varuint length} [({varuint value length} bytes), ...]",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 39,
      "Name": "bytec",
      "Returns": "B",
      "Cost": 1,
      "Size": 2,
      "Doc": "push bytes constant to stack by index into constants",
      "ImmediateNote": "{uint8 byte constant index}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 40,
      "Name": "bytec_0",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 0 from bytecblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 41,
      "Name": "bytec_1",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 1 from bytecblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 42,
      "Name": "bytec_2",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 2 from bytecblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 43,
      "Name": "bytec_3",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push constant 3 from bytecblock to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 44,
      "Name": "arg",
      "Returns": "B",
      "Cost": 1,
      "Size": 2,
      "Doc": "push Args[N] value to stack by index",
      "ImmediateNote": "{uint8 arg index N}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 45,
      "Name": "arg_0",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push Args[0] to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 46,
      "Name": "arg_1",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push Args[1] to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 47,
      "Name": "arg_2",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push Args[2] to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 48,
      "Name": "arg_3",
      "Returns": "B",
      "Cost": 1,
      "Size": 1,
      "Doc": "push Args[3] to stack",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 49,
      "Name": "txn",
      "Returns": ".",
      "Cost": 1,
      "Size": 2,
      "ArgEnum": [
        "Sender",
        "Fee",
        "FirstValid",
        "FirstValidTime",
        "LastValid",
        "Note",
        "Lease",
        "Receiver",
        "Amount",
        "CloseRemainderTo",
        "VotePK",
        "SelectionPK",
        "VoteFirst",
        "VoteLast",
        "VoteKeyDilution",
        "Type",
        "TypeEnum",
        "XferAsset",
        "AssetAmount",
        "AssetSender",
        "AssetReceiver",
        "AssetCloseTo",
        "GroupIndex",
        "TxID"
      ],
      "ArgEnumTypes": "BUUUUBBBUBBBUUUBUUUBBBUB",
      "Doc": "push field from current transaction to stack",
      "DocExtra": "FirstValidTime causes the program to fail. The field is reserved for future use.",
      "ImmediateNote": "{uint8 transaction field index}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 50,
      "Name": "global",
      "Returns": ".",
      "Cost": 1,
      "Size": 2,
      "ArgEnum": [
        "MinTxnFee",
        "MinBalance",
        "MaxTxnLife",
        "ZeroAddress",
        "GroupSize"
      ],
      "ArgEnumTypes": "UUUBU",
      "Doc": "push value from globals to stack",
      "ImmediateNote": "{uint8 global field index}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 51,
      "Name": "gtxn",
      "Returns": ".",
      "Cost": 1,
      "Size": 3,
      "ArgEnum": [
        "Sender",
        "Fee",
        "FirstValid",
        "FirstValidTime",
        "LastValid",
        "Note",
        "Lease",
        "Receiver",
        "Amount",
        "CloseRemainderTo",
        "VotePK",
        "SelectionPK",
        "VoteFirst",
        "VoteLast",
        "VoteKeyDilution",
        "Type",
        "TypeEnum",
        "XferAsset",
        "AssetAmount",
        "AssetSender",
        "AssetReceiver",
        "AssetCloseTo",
        "GroupIndex",
        "TxID"
      ],
      "ArgEnumTypes": "BUUUUBBBUBBBUUUBUUUBBBUB",
      "Doc": "push field to the stack from a transaction in the current transaction group",
      "DocExtra": "for notes on transaction fields available, see `txn`. If this transaction is _i_ in the group, `gtxn i field` is equivalent to `txn field`",
      "ImmediateNote": "{uint8 transaction group index}{uint8 transaction field index}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 52,
      "Name": "load",
      "Returns": ".",
      "Cost": 1,
      "Size": 2,
      "Doc": "copy a value from scratch space to the stack",
      "ImmediateNote": "{uint8 position in scratch space to load from}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 53,
      "Name": "store",
      "Args": ".",
      "Cost": 1,
      "Size": 2,
      "Doc": "pop a value from the stack and store to scratch space",
      "ImmediateNote": "{uint8 position in scratch space to store to}",
      "Groups": [
        "Loading Values"
      ]
    },
    {
      "Opcode": 64,
      "Name": "bnz",
      "Args": "U",
      "Cost": 1,
      "Size": 3,
      "Doc": "branch if value X is not zero",
      "DocExtra": "The `bnz` instruction opcode 0x40 is followed by two immediate data bytes which are a high byte first and low byte second which together form a 16 bit offset which the instruction may branch to. For a bnz instruction at `pc`, if the last element of the stack is not zero then branch to instruction at `pc + 3 + N`, else proceed to next instruction at `pc + 3`. Branch targets must be well aligned instructions. (e.g. Branching to the second byte of a 2 byte op will be rejected.) Branch offsets are currently limited to forward branches only, 0-0x7fff. A future expansion might make this a signed 16 bit integer allowing for backward branches and looping.",
      "ImmediateNote": "{0..0x7fff forward branch offset, big endian}",
      "Groups": [
        "Flow Control"
      ]
    },
    {
      "Opcode": 72,
      "Name": "pop",
      "Args": ".",
      "Cost": 1,
      "Size": 1,
      "Doc": "discard value X from stack",
      "Groups": [
        "Flow Control"
      ]
    },
    {
      "Opcode": 73,
      "Name": "dup",
      "Args": ".",
      "Returns": "..",
      "Cost": 1,
      "Size": 1,
      "Doc": "duplicate last value on stack",
      "Groups": [
        "Flow Control"
      ]
    }
  ]
}
''';
